%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(erl.lang.ets).


-export([all/0, db_delete/1, db_info/2, delete/2, delete_all_objects/1,
	 delete_object/2, first/1, old_fixtable/2, insert/2, last/1,
	 lookup/2, lookup_element/3, match/1, match/2, match/3,
	 match_object/1, match_object/2, match_object/3,
	 match_spec_compile/1, match_spec_run/2, match_spec_run/3,
	 is_key/2, new/2, next/2, prev/2, rename/2, fixtable/2,
	 select/1, select/2, select/3, select_delete/2,
	 select_reverse/1, select_reverse/2, select_reverse/3, slot/2,
	 update_counter/3]).

%% @doc See {@link /stdlib/ets:all/0}.
all() ->
    erl.lang.ets:all().	% Built-in function

%% @doc Undocumented. Do not use.
db_delete(X) ->
    erl.lang.ets:db_delete(X).	% Built-in function

%% @doc Undocumented. Do not use.
db_info(X,Y) ->
    erl.lang.ets:db_info(X,Y).	% Built-in function

%% @doc See {@link /stdlib/ets:delete/2}.
delete(Tab, Key) ->
    erl.lang.ets:delete(Tab, Key).	% Built-in function

%% @doc See {@link /stdlib/ets:delete_all_objects/1}.
delete_all_objects(Tab) ->
    erl.lang.ets:delete_all_objects(Tab).	% Built-in function

%% @doc See {@link /stdlib/ets:delete_object/2}.
delete_object(Tab, Object) ->
    erl.lang.ets:delete_object(Tab, Object).	% Built-in function

%% @doc See {@link /stdlib/ets:first/1}.
first(Tab) ->
    erl.lang.ets:first(Tab).	% Built-in function

% @deprecated Use {@link fixtable/2} instead. See {@link
% /stdlib/ets:fixtable/2} for details.
old_fixtable(Tab, State) ->
    erl.lang.ets:old_fixtable(Tab, State).	% Built-in function

%% @doc See {@link /stdlib/ets:insert/2}.
insert(Tab, Objects) ->
    erl.lang.ets:insert(Tab, Objects).	% Built-in function

%% @doc See {@link /stdlib/ets:last/1}.
last(Tab) ->
    erl.lang.ets:last(Tab).	% Built-in function

%% @doc See {@link /stdlib/ets:lookup/2}.
lookup(Tab, Key) ->
    erl.lang.ets:lookup(Tab, Key).	% Built-in function

%% @doc See {@link /stdlib/ets:lookup_element/3}.
lookup_element(Tab, Key, Pos) ->
    erl.lang.ets:lookup_element(Tab, Key, Pos).	% Built-in function

%% @doc See {@link /stdlib/ets:match/1}.
match(Continuation) ->
    erl.lang.ets:match(Continuation).	% Built-in function

%% @doc See {@link /stdlib/ets:match/2}.
match(Tab, Pattern) ->
    erl.lang.ets:match(Tab, Pattern).	% Built-in function

%% @doc See {@link /stdlib/ets:match/3}.
match(Tab, Pattern, Limit) ->
    erl.lang.ets:match(Tab, Pattern, Limit).	% Built-in function

%% @doc See {@link /stdlib/ets:match_object/1}.
match_object(Continuation) ->
    erl.lang.ets:match_object(Continuation).	% Built-in function

%% @doc See {@link /stdlib/ets:match_object/2}.
match_object(Tab, Pattern) ->
    erl.lang.ets:match_object(Tab, Pattern).	% Built-in function

%% @doc See {@link /stdlib/ets:match_object/3}.
match_object(Tab, Pattern, Limit) ->
    erl.lang.ets:match_object(Tab, Pattern, Limit).	% Built-in function

%% @doc Undocumented. Do not use.
match_spec_compile(X) ->
    erl.lang.ets:match_spec_compile(X).	% Built-in function

%% @doc Undocumented. Do not use.
match_spec_run(X,Y) ->
    erl.lang.ets:match_spec_run(X,Y).	% Built-in function

%% @doc Undocumented. Do not use.
match_spec_run(X,Y,Z) ->
    erl.lang.ets:match_spec_run(X,Y,Z).	% Built-in function

%% @doc See {@link /stdlib/ets:member/2}.
is_key(Tab, Key) ->
    erl.lang.ets:is_key(Tab, Key).	% Built-in function

%% @doc See {@link /stdlib/ets:new/2}.
new(Name, Options) ->
    erl.lang.ets:new(Name, Options).	% Built-in function

%% @doc See {@link /stdlib/ets:next/2}.
next(Tab, Key) ->
    erl.lang.ets:next(Tab, Key).	% Built-in function

%% @doc See {@link /stdlib/ets:prev/2}.
prev(Tab, Key) ->
    erl.lang.ets:prev(Tab, Key).	% Built-in function

%% @doc See {@link /stdlib/ets:prev/2}.
rename(Tab, Name) ->
    erl.lang.ets:rename(Tab, Name).	% Built-in function

%% @doc See {@link /stdlib/ets:safe_fixtable/2}.
fixtable(Tab, State) ->
    erl.lang.ets:fixtable(Tab, State).	% Built-in function

%% @doc See {@link /stdlib/ets:select/1}.
select(Continuation) ->
    erl.lang.ets:select(Continuation).	% Built-in function

%% @doc See {@link /stdlib/ets:select/2}.
select(Tab, MatchSpec) ->
    erl.lang.ets:select(Tab, MatchSpec).	% Built-in function

%% @doc See {@link /stdlib/ets:select/3}.
select(Tab, MatchSpec, Limit) ->
    erl.lang.ets:select(Tab, MatchSpec, Limit).	% Built-in function

%% @doc See {@link /stdlib/ets:select_delete/2}.
select_delete(Tab, MatchSpec) ->
    erl.lang.ets:select_delete(Tab, MatchSpec).	% Built-in function

%% @doc Undocumented. Do not use.
select_reverse(X) ->
    erl.lang.ets:select_reverse(X).	% Built-in function

%% @doc Undocumented. Do not use.
select_reverse(X,Y) ->
    erl.lang.ets:select_reverse(X,Y).	% Built-in function

%% @doc Undocumented. Do not use.
select_reverse(X,Y,Z) ->
    erl.lang.ets:select_reverse(X,Y,Z).	% Built-in function

%% @doc See {@link /stdlib/ets:slot/2}.
slot(Tab, I) ->
    erl.lang.ets:slot(Tab, I).	% Built-in function

%% @doc See {@link /stdlib/ets:update_counter/3}.
update_counter(Tab, Key, Incr) ->
    erl.lang.ets:update_counter(Tab, Key, Incr).	% Built-in function
