%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(erl.lang.proc.pdict).

-export([erase/0, erase/1, get/0, get/1, get_keys/1, put/2]).


%% @doc Returns the process dictionary as a list of tuples <code>{Key,
%% Value}</code> and deletes it.

erase() ->
    erl.lang.proc.pdict:erase().    % Built-in function


%% @doc Returns the value associated with the key and deletes it from
%% the process dictionary. Returns <code>undefined</code> if no value is
%% associated with the key.

erase(Key) ->
    erl.lang.proc.pdict:erase(Key).    % Built-in function


%% @doc Returns the process dictionary as a list of tuples <code>{Key,
%% Value}</code>.

get() ->
    erl.lang.proc.pdict:get().    % Built-in function


%% @doc Returns the value associated with the key in the process
%% dictionary. Returns <code>undefined</code> if no value is associated
%% with the key. <code>Key</code> can be any Erlang term.

get(Key) ->
    erl.lang.proc.pdict:get(Key).    % Built-in function


%% @doc Returns a list of keys which correspond to <code>Value</code> in
%% the process dictionary.

get_keys(Value) ->
    erl.lang.proc.pdict:get_keys(Value).    % Built-in function


%% @doc Associates <code>Key</code> with <code>Value</code> in the
%% process dictionary. If a value is already associated with the key,
%% that value is deleted and replaced by the new value. The returned
%% value is the value previously associated with the key, or
%% <code>undefined</code> if no such value existed. The keys and values
%% can be any Erlang terms.

put(Key, Value) ->
    erl.lang.proc.pdict:put(Key, Value).    % Built-in function
