%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(erl.system).

-export([garbage_collect/0, garbage_collect/1, halt/0, halt/1, info/1,
	 now/0, set_flag/2, statistics/1]).


%% @doc Forces an immediate garbage collection of the currently
%% executing process. You should not use {@link garbage_collect/0}
%% unless you have noticed or have good reasons to suspect that the
%% spontaneous garbage collection will occur too late or not at
%% all. Improper use may seriously degrade system performance.
%%
%% <p>Compatibility note: In versions of OTP prior to R7, the garbage
%% collection took place at the next context switch, not immediately. To
%% force a context switch after a call to {@link garbage_collect/0}, it
%% was sufficient to make any function call.</p>

garbage_collect() ->
    erl.system:garbage_collect().	% Built-in function


%% @doc Works like {@link garbage_collect/0} but on any process. The
%% same caveats apply. Returns false if Pid refers to a dead process;
%% true otherwise.

garbage_collect(Pid) ->
    erl.system:garbage_collect(Pid).	% Built-in function


%% @doc Halts the Erlang runtime system and indicates normal exit to the
%% calling environment. Has no return value.

halt() ->
    erl.system:halt().    % Built-in function


%% @doc Halts the Erlang runtime system. Has no return
%% value. <code>Status</code> must be a non-negative integer, or a
%% string. If <code>Status</code> is an integer, it is returned as an
%% exit status of Erlang to the calling environment. If
%% <code>Status</code> is a string, this function produces an Erlang
%% crash dump using the string as slogan, and then exits with a non-zero
%% status code.
%%
%% <p>Note that on many platforms, only the status codes 0-255 are
%% supported by the operating system.</p>

halt(Status) ->
    erl.system:halt(Status).    % Built-in function


%% @doc See {@link /kernel/erlang:system_info/1}.

info(What) ->
    erl.system:info(What).	% Built-in function


%% @doc Returns the current time. The value is relative to the "epoch"
%% base time, i.e., January 1, 1970, 00:00:00 GMT. The precision is
%% dependent on the host machine. It is guaranteed that subsequent calls
%% to this BIF returns continuously increasing values. Hence, the
%% returned value can be used to generate unique time-stamps.

now() -> erl.system:now().	% Built-in function


%% @doc See {@link /kernel/erlang:system_flag/2}.

set_flag(Flag, Value) ->
    erl.system:set_flag(Flag, Value).	% Built-in function


%% @doc See {@link /kernel/erlang:statistics/1}.

statistics(Type) ->
    erl.system:statistics(Type).	% Built-in function
