%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id: format_SUITE.erl,v 1.1.1.1 2004/06/18 12:10:14 tobiasl Exp $
%%

-module(format_SUITE).
-author('ingela@erix.ericsson.se').

-include("test_server.hrl").

%% Test server specific exports
-export([all/1, init_per_testcase/2, end_per_testcase/2]).

%% Test cases must be exported.
-export([base_64_encode/1, base_64_decode/1]).


all(doc) ->
    ["Test library functions to the http client and server."];
all(suite) ->
    [base_64_encode, base_64_decode].
 
init_per_testcase(_Case, Config) ->
    ?line Dog = test_server:timetrap(?t:minutes(1)),
    ?line NewConfig = lists:keydelete(watchdog, 1, Config),
    [{watchdog, Dog} | NewConfig].

end_per_testcase(Case, Config) ->
    Dog = ?config(watchdog, Config),
    test_server:timetrap_cancel(Dog),
    ok.

%-------------------------------------------------------------------------
% Test cases starts here.
%-------------------------------------------------------------------------

base_64_encode(doc) ->
    ["Test base_64:encode/1."];
base_64_encode(suite) ->
    [];
base_64_encode(Config) when list(Config) ->
    %% Two pads 
    "QWxhZGRpbjpvcGVuIHNlc2FtZQ==" =
	http_base_64:encode("Aladdin:open sesame"),
    %% One pad
    "SGVsbG8gV29ybGQ=" = http_base_64:encode("Hello World"),
    %% No pad
    "QWxhZGRpbjpvcGVuIHNlc2Ft" = http_base_64:encode("Aladdin:open sesam"),
    ok.

base_64_decode(doc) ->
    ["Test base_64:decode/1."];
base_64_decode(suite) ->
    [];
base_64_decode(Config) when list(Config) ->
    %% Two pads 
    "Aladdin:open sesame" = 
	http_base_64:decode("QWxhZGRpbjpvcGVuIHNlc2FtZQ=="),
    %% One pad
    "Hello World" = http_base_64:decode("SGVsbG8gV29ybGQ="),
    %% No pad
    "Aladdin:open sesam" = http_base_64:decode("QWxhZGRpbjpvcGVuIHNlc2Ft"),
    ok.



