/* ``The contents of this file are subject to the Erlang Public License,
 * Version 1.1, (the "License"); you may not use this file except in
 * compliance with the License. You should have received a copy of the
 * Erlang Public License along with this software. If not, it can be
 * retrieved via the world wide web at http://www.erlang.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Original Code is Ericsson Utvecklings AB.
 * Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
 * AB. All Rights Reserved.''
 * 
 *     $Id: NodePing.java,v 1.1.1.1 2004/07/05 09:15:14 tobiasl Exp $
 */

import com.ericsson.otp.erlang.*;

class NodePing {
    
    /*
      Implements test case jinterface_SUITE:node_ping/1

      Creates three OtpNode objects. One with default cookie, one with
      specified same cookie as the node running the test case and one
      with a faulty cookie. From each OtpNode object the test_server
      node is pinged.

      Also the default cookie node pings itself, and the node with the
      specified cookie pings the node with default cookie.
    */

    public static void main(String argv[]) {

	String cookie = argv[0];
	String erlNode = argv[1];

	try {
	    OtpNode node1 = new OtpNode("javanode1");
	    ping(node1,erlNode,"Default cookie:",true,1);
	    ping(node1,node1.node(),"Self:",true,2);
	    ping(node1,"javanode1","Self:",true,3);        // hangs!!

	    OtpNode node2 = new OtpNode("javanode2",cookie);
	    ping(node2,erlNode,"Specified cookie:",true,4);
	    ping(node2,"javanode1","Javanode (no host):",true,5);
	    ping(node2,node1.node(),"Javanode:",true,6);

	    OtpNode node3 = new OtpNode("javanode3","faultycookie");
	    ping(node3,erlNode,"Faulty cookie:",false,7);
	}
	catch (Exception e) {
	    fail("" + e, 8);
	}
    }
    
    private static void ping(OtpNode node, String remote, String descr, 
			     boolean expected, int reason) {
	if ( node.ping(remote,2000) == expected ) {
	    System.out.println(descr + " ping(" + remote + ") -> " + expected);
	}
	else {
	    fail("ERROR: " + descr + " ping(" + remote +") -> " + !expected,
		 reason);
	}
    }
    
    private static void fail(String str, int reason) {
	System.out.println(str);
	System.exit(reason);
    }
}
