%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id: nc_SUITE.erl,v 1.1.1.2 2004/07/05 09:15:14 tobiasl Exp $
%%
-module(nc_SUITE).


-include("test_server.hrl").


-export([all/1,
	 init_per_suite/1,
	 end_per_suite/1,
	 init_per_testcase/2,
	 end_per_testcase/2]).

-export([pid_roundtrip/1,
	 port_roundtrip/1,
	 ref_roundtrip/1,
	 require_xpid_xport_connect/1,
	 r7_compat_connect/1]).


%% Top of cases

all(doc) -> [];
all(suite) -> [pid_roundtrip,
	       port_roundtrip,
	       ref_roundtrip,
	       require_xpid_xport_connect,
	       r7_compat_connect].

init_per_suite(Config) when list(Config) ->
    jitu:init_all(Config).

end_per_suite(Config) ->
    jitu:finish_all(Config).



%% Add/remove watchdog before/after each test case.
%%
init_per_testcase(_Case, Config) ->
    WatchDog = test_server:timetrap(test_server:seconds(20)),
    [{watchdog, WatchDog}| Config].

end_per_testcase(_Case, Config) ->
    WatchDog = ?config(watchdog, Config),
    test_server:timetrap_cancel(WatchDog).


%%
%% Test cases
%%

pid_roundtrip(doc) -> [];
pid_roundtrip(suite) -> [];
pid_roundtrip(Config) when list(Config)->
    ?line ThisNode = {node(), erlang:system_info(creation)},
    ?line RemNode = {gurka@sallad, 2},
    ?line do_echo([self(),
		   mk_pid(ThisNode, 4711, 4711),
		   mk_pid(ThisNode, 32767, 8191),
		   mk_pid(RemNode, 4711, 4711),
		   mk_pid(RemNode, 32767, 8191)],
		  Config).

port_roundtrip(doc) -> [];
port_roundtrip(suite) -> [];
port_roundtrip(Config) when list(Config)->
    ?line ThisNode = {node(), erlang:system_info(creation)},
    ?line RemNode = {gurka@sallad, 2},
    ?line do_echo([hd(erlang:ports()),
		   mk_port(ThisNode, 4711),
		   mk_port(ThisNode, 268435455),
		   mk_port(RemNode, 4711),
		   mk_port(RemNode, 268435455)],
		  Config).

ref_roundtrip(doc) -> [];
ref_roundtrip(suite) -> [];
ref_roundtrip(Config) when list(Config)->
    ?line ThisNode = {node(), erlang:system_info(creation)},
    ?line RemNode = {gurka@sallad, 2},
    ?line do_echo([make_ref(),
		   mk_ref(ThisNode, [4711]),
		   mk_ref(ThisNode, [4711, 4711, 4711]),
		   mk_ref(ThisNode, [262143, 4294967295, 4294967295]),
		   mk_ref(RemNode, [4711]),
		   mk_ref(RemNode, [4711, 4711, 4711]),
		   mk_ref(RemNode, [262143, 4294967295, 4294967295])],
		   Config).

do_echo(DataList, Config) when list(DataList), list(Config) ->
    ?line Name = make_name(),
    ?line true = register(Name, self()),
    ?line JName = make_name(),
    ?line spawn_link(fun () ->
			     ?line ok = jitu:java(?config(java, Config),
						  ?config(data_dir, Config),
						  "echo_server",
						  [JName,
						   erlang:get_cookie(),
						   node(),
						   Name]),
			     ?line Name ! {done, JName}
		     end),
    ?line receive
	      {echo_server, JName, Pid} ->
		  ?line ?t:format("echo_server: ~p (~p)~n",
				  [Pid, node(Pid)]),
		  ?line ?t:format("nodes(hidden): ~p~n",
				  [nodes(hidden)]),
		  ?line do_echo(DataList, Pid)
	  end,
    ?line Pid ! bye,
    ?line receive
	      {done, JName} ->
		  ?line ok
	  end;

do_echo([D|Ds], Echoer) when pid(Echoer) ->
    ?line Echoer ! {self(), D},
    ?line receive
	      {Echoer, Data} ->
		  ?line D = Data
	  end,
    ?line do_echo(Ds, Echoer);
do_echo([], _) ->
    ok.


require_xpid_xport_connect(doc) -> [];
require_xpid_xport_connect(suite) -> [];
require_xpid_xport_connect(Config) when list(Config) ->
    ?line maybe_run_conn_test(Config, false).

r7_compat_connect(doc) -> [];
r7_compat_connect(suite) -> [];
r7_compat_connect(Config) when list(Config) ->
    ?line maybe_run_conn_test(Config, true).

maybe_run_conn_test(Config, R7Compat) ->
    case ?t:os_type() of
	{unix, sunos} ->
	    case erlang:system_info(compat_rel) of
		7 ->
		    ?line run_conn_test(Config, R7Compat);
		_ ->
		    ?line Pa = filename:dirname(code:which(?MODULE)),
		    ?line ?t:run_on_shielded_node(fun () ->
							  run_conn_test(Config,
									R7Compat)
						  end,
						  "+R7 -pa " ++ Pa)
	    end;
	_ ->
	    ?line {skipped, "Only run on {unix, sunos}"}
    end.

run_conn_test(Config, R7Compat) when R7Compat == true; R7Compat == false ->
    ?line Name = make_name(),
    ?line true = register(Name, self()),
    ?line JName = make_name(),
    ?line spawn_link(fun () ->
			     ?line ok = jitu:java(?config(java, Config),
						  ?config(data_dir, Config),
						  "connection_server",
						  [JName,
						   erlang:get_cookie(),
						   node(),
						   Name],
						  case R7Compat of
						      true ->
							  "-DOtpCompatRel=7";
						      false ->
							  ""
						  end),
			     ?line Name ! {done, JName}
		     end),
    ?line Pa = filename:dirname(code:which(?MODULE)),
    ?line {ok, This} = ?t:start_node(make_name(),
				     peer,
				     [{args, "-pa "++Pa}]),
    ?line {ok, R7} = ?t:start_node(make_name(),
				   peer,
				   [{args, " -pa " ++ Pa},
				    {erl, [{release, "r7b01"}]}]),
    ?line CServ = receive
		      {JName, Pid} ->
			  ?line ?t:format("connection_server: ~p (~p)~n",
					  [Pid, node(Pid)]),
			  ?line ?t:format("nodes(hidden): ~p~n",
					  [nodes(hidden)]),
			  ?line Pid
		  end,
    ?line JI = node(CServ),
    ?line pong = rpc:call(This, net_adm, ping, [JI]),
    ?line true = rpc:call(This, net_kernel, disconnect, [JI]),
    ?line R7PingRes = case R7Compat of
			  true -> ?line pong;
			  false -> ?line pang
		      end,
    ?line R7PingRes = rpc:call(R7, net_adm, ping, [JI]),
    ?line CServ ! {self(), This},
    ?line receive
	      {CServ, This, ThisConnected} ->
		  ?line true = ThisConnected
	  end,
    ?line CServ ! {self(), R7},
    ?line receive
	      {CServ, R7, R7Connected} ->
		  ?line R7Compat = R7Connected
	  end,
    ?line ?t:stop_node(This),
    ?line ?t:stop_node(R7),
    ?line CServ ! bye,
    ?line receive
	      {done, JName} ->
		  ?line ok
	  end,
    ?line ok.


%%
%% Utils...
%%

make_name() ->
    ?line {A, B, C} = now(),
    ?line list_to_atom(atom_to_list(?MODULE)
		       ++ "-" ++ integer_to_list(A)
		       ++ "-" ++ integer_to_list(B)
		       ++ "-" ++ integer_to_list(C)).



-define(VERSION_MAGIC,       131).

-define(ATOM_EXT,            100).
-define(REFERENCE_EXT,       101).
-define(PORT_EXT,            102).
-define(PID_EXT,             103).
-define(NEW_REFERENCE_EXT,   114).

uint32_be(Uint) when integer(Uint), 0 =< Uint, Uint < 1 bsl 32 ->
    [(Uint bsr 24) band 16#ff,
     (Uint bsr 16) band 16#ff,
     (Uint bsr 8) band 16#ff,
     Uint band 16#ff];
uint32_be(Uint) ->
    exit({badarg, uint32_be, [Uint]}).


uint16_be(Uint) when integer(Uint), 0 =< Uint, Uint < 1 bsl 16 ->
    [(Uint bsr 8) band 16#ff,
     Uint band 16#ff];
uint16_be(Uint) ->
    exit({badarg, uint16_be, [Uint]}).

uint8(Uint) when integer(Uint), 0 =< Uint, Uint < 1 bsl 8 ->
    Uint band 16#ff;
uint8(Uint) ->
    exit({badarg, uint8, [Uint]}).



mk_pid({NodeName, Creation}, Number, Serial) when atom(NodeName) ->
    mk_pid({atom_to_list(NodeName), Creation}, Number, Serial);
mk_pid({NodeName, Creation}, Number, Serial) ->
    case catch binary_to_term(list_to_binary([?VERSION_MAGIC,
					?PID_EXT,
					?ATOM_EXT,
					uint16_be(length(NodeName)),
					NodeName,
					uint32_be(Number),
					uint32_be(Serial),
					uint8(Creation)])) of
	Pid when pid(Pid) ->
	    Pid;
	{'EXIT', {badarg, _}} ->
	    exit({badarg, mk_pid, [{NodeName, Creation}, Number, Serial]});
	Other ->
	    exit({unexpected_binary_to_term_result, Other})
    end.

mk_port({NodeName, Creation}, Number) when atom(NodeName) ->
    mk_port({atom_to_list(NodeName), Creation}, Number);
mk_port({NodeName, Creation}, Number) ->
    case catch binary_to_term(list_to_binary([?VERSION_MAGIC,
					      ?PORT_EXT,
					      ?ATOM_EXT,
					      uint16_be(length(NodeName)),
					      NodeName,
					      uint32_be(Number),
					      uint8(Creation)])) of
	Port when port(Port) ->
	    Port;
	{'EXIT', {badarg, _}} ->
	    exit({badarg, mk_port, [{NodeName, Creation}, Number]});
	Other ->
	    exit({unexpected_binary_to_term_result, Other})
    end.

mk_ref({NodeName, Creation}, Numbers) when atom(NodeName),
					   integer(Creation),
					   list(Numbers) ->
    mk_ref({atom_to_list(NodeName), Creation}, Numbers);
mk_ref({NodeName, Creation}, [Number]) when list(NodeName),
					    integer(Creation),
					    integer(Number) ->
    case catch binary_to_term(list_to_binary([?VERSION_MAGIC,
					      ?REFERENCE_EXT,
					      ?ATOM_EXT,
					      uint16_be(length(NodeName)),
					      NodeName,
					      uint32_be(Number),
					      uint8(Creation)])) of
	Ref when reference(Ref) ->
	    Ref;
	{'EXIT', {badarg, _}} ->
	    exit({badarg, mk_ref, [{NodeName, Creation}, [Number]]});
	Other ->
	    exit({unexpected_binary_to_term_result, Other})
    end;
mk_ref({NodeName, Creation}, Numbers) when list(NodeName),
					   integer(Creation),
					   list(Numbers) ->
    case catch binary_to_term(list_to_binary([?VERSION_MAGIC,
					      ?NEW_REFERENCE_EXT,
					      uint16_be(length(Numbers)),
					      ?ATOM_EXT,
					      uint16_be(length(NodeName)),
					      NodeName,
					      uint8(Creation),
					      lists:map(fun (N) ->
								uint32_be(N)
							end,
							Numbers)])) of
	Ref when reference(Ref) ->
	    Ref;
	{'EXIT', {badarg, _}} ->
	    exit({badarg, mk_ref, [{NodeName, Creation}, Numbers]});
	Other ->
	    exit({unexpected_binary_to_term_result, Other})
    end.
